/*
 * Copyright (C) 2006 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */
/*
 * based in parts on libfaad, Copyright (C) 2003-2005 M. Bakker,
 * Ahead Software AG
 */

/** \file aac_common.c common functions of aac_afh and aadcec */

#include "para.h"
#include "aac.h"
#include "error.h"

/**
 * get a new libfaad decoder handle
 *
 */
NeAACDecHandle aac_open(void)
{
	NeAACDecHandle h = NeAACDecOpen();
	NeAACDecConfigurationPtr c = NeAACDecGetCurrentConfiguration(h);

	c->defObjectType = LC;
	c->outputFormat = FAAD_FMT_16BIT;
	c->downMatrix = 0;
	NeAACDecSetConfiguration(h, c);
	return h;
};

static int aac_read_decoder_length(unsigned char *buf, int *description_len)
{
	uint8_t b;
	uint8_t numBytes = 0;
	uint32_t length = 0;

	do {
		b = buf[numBytes];
		numBytes++;
		length = (length << 7) | (b & 0x7F);
	} while
		((b & 0x80) && numBytes < 4);
	*description_len = numBytes;
	return length;
}

/**
 * search for the position and the length of the decoder configuration
 *
 * \param buf buffer to seach
 * \param buflen length of \a buf
 * \param skip Upon succesful return, this contains the offset in \a buf where
 * the decoder config starts.
 *
 * \return The length of the decoder configuration
 */
ssize_t aac_find_esds(unsigned char *buf, size_t buflen, size_t *skip)
{
	size_t i;

	for (i = 0; i + 4 < buflen; i++) {
		unsigned char *p = buf + i;
		int decoder_length, description_len;

		if (p[0] != 'e' || p[1] != 's' || p[2] != 'd' || p[3] != 's')
			continue;
		i += 8;
		p = buf + i;
		PARA_INFO_LOG("found esds@%zu, next: %x\n", i, *p);
		if (*p == 3)
			i += 8;
		else
			i += 6;
		p = buf + i;
		PARA_INFO_LOG("next: %x\n", *p);
		if (*p != 4)
			continue;
		i += 18;
		p = buf + i;
		PARA_INFO_LOG("next: %x\n", *p);
		if (*p != 5)
			continue;
		i++;
		p = buf + i;
		decoder_length = aac_read_decoder_length(p, &description_len);
		PARA_INFO_LOG("decoder length: %d\n", decoder_length);
		i += description_len;
		*skip = i;
		return decoder_length;
	}
	return -E_ESDS;
}

/**
 * search for the first entry in the stco table
 *
 * \param buf buffer to seach
 * \param buflen length of \a buf
 * \param skip Upon succesful return, this contains the number
 * of bytes to skip from the input buffer.
 *
 * \return the position of the first entry in the table on success,
 * -E_STCO on errors.
 */
ssize_t aac_find_entry_point(unsigned char *buf, size_t buflen, size_t *skip)
{
	ssize_t ret;
	size_t i;

	for (i = 0; i + 20 < buflen; i++) {
		unsigned char *p = buf + i;

		if (p[0] != 's' || p[1] != 't' || p[2] != 'c' || p[3] != 'o')
			continue;
		PARA_INFO_LOG("found stco@%zu\n", i);
		i += 12;
		ret = aac_read_int32(buf + i); /* first offset */
		i += 4;
		PARA_INFO_LOG("entry point: %zd\n", ret);
		*skip = i;
		return ret;
	}
	PARA_WARNING_LOG("stco not found, buflen: %zu\n", buflen);
	return -E_STCO;
}
