/*
 * Copyright (C) 2005-2006 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

/** \file mp3dec.c paraslash's mp3 decoder */

#include "para.h"
#include "list.h"
#include "sched.h"
#include "filter.h"
#include "error.h"
#include <mad.h>
#include "string.h"

/** the output buffer size */
#define MP3_OUTBUF_SIZE 128 * 1024

/** \cond a helper macro */
#define MAD_TO_SHORT(f) (f) >= MAD_F_ONE?  SHRT_MAX :\
	(f) <= -MAD_F_ONE? -SHRT_MAX : (signed short) ((f) >> (MAD_F_FRACBITS - 15))
/** \endcond */

/**
 * data specific to the mp3dec filter
 *
 * \sa filter, filter_node
 */
struct private_mp3dec_data {
	/** information on the current mp3 stream */
	struct mad_stream stream;
	/** information about the frame which is currently decoded */
	struct mad_frame frame;
	/** contains the PCM output */
	struct mad_synth synth;
};

/* TODO: Convert all input if possible */
#define FRAME_HEADER_SIZE 4
static ssize_t mp3dec(char *inbuffer, size_t len, struct filter_node *fn)
{
	int i, ret;
	struct private_mp3dec_data *pmd = fn->private_data;
	size_t copy = PARA_MIN(len, 4096);

	if (fn->loaded > fn->bufsize * 4 / 5)
		return 0;
	mad_stream_buffer(&pmd->stream, (unsigned char *) inbuffer, copy);
	pmd->stream.error = 0;
next_frame:
	ret = mad_header_decode(&pmd->frame.header, &pmd->stream);
	if (ret < 0) {
		if (!MAD_RECOVERABLE(pmd->stream.error))
			goto out;
		return FRAME_HEADER_SIZE;
	}
	fn->fc->samplerate = pmd->frame.header.samplerate;
	fn->fc->channels = MAD_NCHANNELS(&pmd->frame.header);
	ret = mad_frame_decode(&pmd->frame, &pmd->stream);
	if (ret) {
		if (MAD_RECOVERABLE(pmd->stream.error) || pmd->stream.error == MAD_ERROR_BUFLEN)
			goto out;
		PARA_ERROR_LOG("fatal: ret = %d, loaded = %zd\n", ret, fn->loaded);
		return -E_MAD_FRAME_DECODE;
	}
	mad_synth_frame(&pmd->synth, &pmd->frame);

	for (i = 0; i < pmd->synth.pcm.length; i++) {
		/* output format: unsigned 16 bit little endian */
		signed short s = MAD_TO_SHORT(pmd->synth.pcm.samples[0][i]);
		fn->buf[fn->loaded++] = s & 0xff;
		fn->buf[fn->loaded++] = s >> 8;
		if (MAD_NCHANNELS(&pmd->frame.header) == 2) { /* stereo */
			s = MAD_TO_SHORT(pmd->synth.pcm.samples[1][i]);
			fn->buf[fn->loaded++] = s & 0xff;
			fn->buf[fn->loaded++] = s >> 8;
		}
		if (fn->loaded != fn->bufsize) /* output buffer not full */
			continue;
		PARA_ERROR_LOG("output buffer full: %zd\n", fn->loaded);
			return -E_MP3DEC_OVERRUN;
	}
	if (fn->loaded <= fn->bufsize * 4 / 5)
		goto next_frame;
out:
	if (pmd->stream.next_frame) { /* we still have some data */
		size_t off = pmd->stream.bufend - pmd->stream.next_frame;
//		PARA_INFO_LOG("off: %zd, rate: %u, returning %zd\n", off,
//			fn->fc->samplerate, copy - off);
		return copy - off;
	}
	return copy;
}

static void mp3dec_close(struct filter_node *fn)
{
	struct private_mp3dec_data *pmd = fn->private_data;

	mad_synth_finish(&pmd->synth);
	mad_frame_finish(&pmd->frame);
	mad_stream_finish(&pmd->stream);

	free(fn->buf);
	fn->buf = NULL;
	free(pmd);
	fn->private_data = NULL;
}

static void mp3dec_open(struct filter_node *fn)
{
	fn->private_data = para_calloc(sizeof(struct private_mp3dec_data));
	struct private_mp3dec_data *pmd = fn->private_data;

	mad_stream_init(&pmd->stream);
	mad_frame_init(&pmd->frame);
	mad_synth_init(&pmd->synth);
	fn->loaded = 0;
	fn->bufsize = MP3_OUTBUF_SIZE;
	fn->buf = para_calloc(fn->bufsize);
}
/**
 * the init function of the mp3dec filter
 *
 * \sa filter::init
 */
void mp3dec_init(struct filter *f)
{
	f->open = mp3dec_open;
	f->convert = mp3dec;
	f->close = mp3dec_close;
}
